class Fdarchivo
  require 'fdarchivo/fdpath'
  require 'fdarchivo/fdcloud'
  require 'fdarchivo/archivo'
  require 'fdarchivo/configuration'
  class << self
    attr_accessor :configuration
  end
  attr_accessor :archivo,
                :doc,
                :path

  attr_accessor :tmp_upload_code

  public
  def initialize(data)
    @doc = nil
    @tmp_upload_code = nil
    @archivo = get_file(data)

    self.path = get_path
  end

  def self.tmp_multi_upload(paths, owner = nil)
    code = self.gen_upload_code
    paths.each do |path|
      a = self.new(path)
      a.tmp_upload(owner, code: code)
    end
    code
  end
  def upload(owner, extra: {})
    self.doc = self.create(owner: owner, extra: extra)
    self.archivo = Fdcloud.new.upload(self.path, self.doc)
    self.doc.update(public_url: archivo.url)
  end
  def tmp_upload(owner = nil, code: self.class.gen_upload_code)
    self.tmp_upload_code = code
    self.doc = create(owner: owner, tmp: true)
    self.archivo = Fdcloud.new.tmp_upload(self.path, self.doc)
    self.doc.update(public_url: archivo.url)
    self.doc.upload_code
  end
  def create(owner: nil, tmp: false, extra: {})
    Archivo.create!(
        {
            path: self.path,
            tipo_subida: self.class,
            owner: owner,
            tmp: tmp,
            upload_code: tmp_upload_code_to_s
        }.merge(extra.to_h))
  end
  def reclamar
    self.doc.reclamar
  end

  def delete
    self.doc.destroy
  end

  def self.reclamar(ids: nil, code: nil, owner: nil, old_owner: nil)
    if ids
      Archivo.reclamar_by_ids(ids, owner: owner).each do |a|
        self.new(a._id.to_s)
      end
    elsif code && owner
      Archivo.reclamar(code, old_owner, owner)
    end
  end
  def self.find_by(ids: [], code: nil, owner: nil)
    Archivo.where(:$or => [
        {:_id.in => ids},
        {upload_code: code, owner: owner}
    ])
  end
  def self.configure
    self.configuration ||= Configuration.new
    yield(configuration)
  end

  protected
  def get_file(id)
    self.doc = Archivo.find(id) if id.is_a?(String)
    # Fixme: Tipo de subida
    if self.doc&.tipo_subida
      Fdcloud.new(self.doc.path)
    else
      id
    end
  end
  def get_path
    begin
      self.doc&.path || self.archivo&.path
    rescue
      self.archivo
    end
  end
  def self.gen_upload_code
    rnd = Random.new
    code = []
    6.times do
      code << rnd.rand(0..9)
    end
    code
  end

  def tmp_upload_code_to_s
    "#{tmp_upload_code[0]}#{tmp_upload_code[1]}#{tmp_upload_code[2]}#{tmp_upload_code[3]}#{tmp_upload_code[4]}#{tmp_upload_code[5]}" unless tmp_upload_code.nil?
  end
end
