class Archivo
  include Mongoid::Document
  include Mongoid::Timestamps

  TIPOS = [
      :IMG, # Imagen
      :DOC, # Documento
      :ARC  # Archivo
  ]

  field :path, type: String
  field :public_url, type: String
  field :extension, type: String
  field :upload_code, type: String
  field :tmp, type: Boolean, default: false
  field :tipo, type: Symbol
  field :tipo_subida, type: String
  field :bucket, type: String
  field :logo, type: Boolean, default: false

  # validates_each :tipo do |doc, attr, value|
  #   doc.errors.add attr,
  #                  "#{value} no esta permitido. Posibles tipos: #{TIPOS}" unless TIPOS.include?(doc.tipo)
  # end
  before_save do |doc|
    if doc.logo
      self.class.where(
          :_id.ne => doc._id,
          owner_id: doc.owner._id,
          logo: true
      ).destroy_all
    end
    doc.extension = doc.path.split('.').last
    doc.tipo = check_tipo(doc.extension)
    if doc.tmp && doc.owner.blank?
      doc.path = "#{doc._id.to_s}.#{doc.extension}"
    else
      doc.path = "#{doc.owner.class.to_s.underscore}/#{doc.owner._id.to_s}/#{doc._id.to_s}.#{doc.extension}"
    end
  end

  before_destroy do |doc|
    Fdarchivo.new(doc._id.to_s).archivo.try(:delete)
  end


  belongs_to :owner,
             polymorphic: true,
             required: false

  def self.reclamar_by_ids(ids, owner: nil)
    self.where(:_id.in => ids).collect do |doc|
      doc.reclamar(owner: owner)
    end
  end
  def self.reclamar(code, old_owner, owner)
    ap self.where(tmp: true, upload_code: code, owner: old_owner)
    self.where(tmp: true, upload_code: code, owner: old_owner).collect do |doc|
      doc.reclamar(owner: owner)
    end
  end

  def reclamar(owner: nil)
    cloud = Fdcloud.new(self.path, tmp: true)
    self.update!(
        tmp: false,
        owner: owner
    )
    self.update!(
        public_url: cloud.move_tmp(self).public_url
    )
    self
  end

  private
  def check_tipo(ext)
    case ext
      when 'jpg', 'jpeg'
        :IMG
      else
        :ARC
    end
  end

end